@@ -1,9 +1,10 @@
 package Net::DNS::Resolver;
 
-#
-# $Id: Resolver.pm 1714 2018-09-21 14:14:55Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1714 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: Resolver.pm 1818 2020-10-18 15:24:42Z willem $)[2];
+
 
 =head1 NAME
 
@@ -12,10 +13,7 @@ Net::DNS::Resolver - DNS resolver class
 =cut
 
 
-use strict;
-use warnings;
-
-use constant CONFIG => defined eval "require Net::DNS::Resolver::$^O";
+use constant CONFIG => defined eval "require Net::DNS::Resolver::$^O";	  ## no critic
 
 use constant OS_CONF => join '::', __PACKAGE__, CONFIG ? $^O : 'UNIX';
 
@@ -31,7 +29,7 @@ __END__
 
     use Net::DNS;
 
-    $resolver = new Net::DNS::Resolver();
+    $resolver = Net::DNS::Resolver->new();
 
     # Perform a lookup, using the searchlist if appropriate.
     $reply = $resolver->search( 'example.com' );
@@ -43,7 +41,7 @@ __END__
     $reply = $resolver->send( 'example.com', 'MX', 'IN' );
 
     # Send a prebuilt query packet
-    $query = new Net::DNS::Packet( ... );
+    $query = Net::DNS::Packet->new( ... );
     $reply = $resolver->send( $query );
 
 =head1 DESCRIPTION
@@ -58,14 +56,14 @@ recursion is desired, etc.
 =head2 new
 
     # Use the default configuration
-    $resolver = new Net::DNS::Resolver();
+    $resolver = Net::DNS::Resolver->new();
 
     # Use my own configuration file
-    $resolver = new Net::DNS::Resolver( config_file => '/my/dns.conf' );
+    $resolver = Net::DNS::Resolver->new( config_file => '/my/dns.conf' );
 
     # Set options in the constructor
-    $resolver = new Net::DNS::Resolver(
-	nameservers => [ '10.1.1.128', '10.1.2.128' ],
+    $resolver = Net::DNS::Resolver->new(
+	nameservers => [ '2001:DB8::1', 'ns.example.com' ],
 	recurse	    => 0,
 	debug	    => 1
 	);
@@ -122,7 +120,7 @@ interfaces may confuse L<Net::DNS>.
 
 
     # Use my own configuration file
-    $resolver = new Net::DNS::Resolver( config_file => '/my/dns.conf' );
+    $resolver = Net::DNS::Resolver->new( config_file => '/my/dns.conf' );
 
 You can include a configuration file of your own when creating a
 resolver object.  This is supported on both Unix and Windows.
@@ -132,8 +130,8 @@ all other configuration files and environment variables are ignored.
 
 
     # Set options in the constructor
-    $resolver = new Net::DNS::Resolver(
-	nameservers => [ '10.1.1.128', '10.1.2.128' ],
+    $resolver = Net::DNS::Resolver->new(
+	nameservers => [ '2001:DB8::1', 'ns.example.com' ],
 	recurse	    => 0
 	);
 
@@ -151,15 +149,14 @@ Prints the resolver state on the standard output.
 
 =head2 query
 
-    $packet = $resolver->query( 'mailhost' );
-    $packet = $resolver->query( 'mailhost.example.com' );
-    $packet = $resolver->query( '192.0.2.1' );
+    $packet = $resolver->query( 'host' );
+    $packet = $resolver->query( 'host.example.com' );
+    $packet = $resolver->query( '2001:DB8::1' );
     $packet = $resolver->query( 'example.com', 'MX' );
     $packet = $resolver->query( 'annotation.example.com', 'TXT', 'IN' );
 
 Performs a DNS query for the given name; the search list is not applied.
-If C<defnames> is true, and the number of dots is less than C<ndots>,
-the default domain will be appended unless name is absolute.
+If C<defnames> is true, the default domain will be appended to unqualified names.
 
 The record type and class can be omitted; they default to A and IN.
 If the name looks like an IP address (IPv4 or IPv6),
@@ -174,19 +171,19 @@ any answers or not, use the C<send()> method instead.
 
 =head2 search
 
-    $packet = $resolver->search( 'mailhost' );
-    $packet = $resolver->search( 'mailhost.example.com' );
-    $packet = $resolver->search( '192.0.2.1' );
+    $packet = $resolver->search( 'host' );
+    $packet = $resolver->search( 'host.example.com' );
+    $packet = $resolver->search( '2001:DB8::1' );
     $packet = $resolver->search( 'example.com', 'MX' );
     $packet = $resolver->search( 'annotation.example.com', 'TXT', 'IN' );
 
 Performs a DNS query for the given name, applying the searchlist if
 appropriate.  The search algorithm is as follows:
 
-Unless the number of dots is less than C<ndots>,
+If the name contains one or more non-terminal dots,
 perform an initial query using the unmodified name.
 
-If C<dnsrch> is true and the name has no terminal dot,
+If the number of dots is less than C<ndots>, and there is no terminal dot,
 try appending each suffix in the search list.
 
 The record type and class can be omitted; they default to A and IN.
@@ -204,8 +201,8 @@ any answers or not, use the C<send()> method instead.
 
     $packet = $resolver->send( $query );
 
-    $packet = $resolver->send( 'mailhost.example.com' );
-    $packet = $resolver->query( '192.0.2.1' );
+    $packet = $resolver->send( 'host.example.com' );
+    $packet = $resolver->send( '2001:DB8::1' );
     $packet = $resolver->send( 'example.com', 'MX' );
     $packet = $resolver->send( 'annotation.example.com', 'TXT', 'IN' );
 
@@ -289,8 +286,8 @@ Here is the example above, implemented using an iterator:
 
     $handle = $resolver->bgsend( $packet ) || die $resolver->errorstring;
 
-    $handle = $resolver->bgsend( 'mailhost.example.com' );
-    $handle = $resolver->bgsend( '192.0.2.1' );
+    $handle = $resolver->bgsend( 'host.example.com' );
+    $handle = $resolver->bgsend( '2001:DB8::1' );
     $handle = $resolver->bgsend( 'example.com', 'MX' );
     $handle = $resolver->bgsend( 'annotation.example.com', 'TXT', 'IN' );
 
@@ -344,16 +341,6 @@ Truncated UDP packets will be retried transparently using TCP while
 continuing to assert busy to the caller.
 
 
-=head2 bgisready
-
-    until ($resolver->bgisready($handle)) {
-	...
-    }
-
-C<bgisready()> is the logical complement of C<bgbusy()> which is retained
-for backward compatibility.
-
-
 =head2 debug
 
     print 'debug flag: ', $resolver->debug, "\n";
@@ -409,7 +396,7 @@ The default is false.
 =head2 nameserver, nameservers
 
     @nameservers = $resolver->nameservers();
-    $resolver->nameservers( '192.0.2.1', '192.0.2.2', '2001:DB8::3' );
+    $resolver->nameservers( '2001:DB8::1', '192.0.2.1' );
     $resolver->nameservers( 'ns.domain.example.' );
 
 Gets or sets the nameservers to be queried.
@@ -493,7 +480,7 @@ Gets or sets the resolver search list.
 
 =head2 srcaddr
 
-    $resolver->srcaddr('192.0.2.1');
+    $resolver->srcaddr('2001::DB8::1');
 
 Sets the source address from which queries are sent.
 Convenient for forcing queries from a specific interface on a
@@ -651,11 +638,11 @@ the resolver:
 =head2 RES_NAMESERVERS
 
     # Bourne Shell
-    RES_NAMESERVERS="192.0.2.1 192.0.2.2 2001:DB8::3"
+    RES_NAMESERVERS="2001:DB8::1 192.0.2.1"
     export RES_NAMESERVERS
 
     # C Shell
-    setenv RES_NAMESERVERS "192.0.2.1 192.0.2.2 2001:DB8::3"
+    setenv RES_NAMESERVERS "2001:DB8::1 192.0.2.1"
 
 A space-separated list of nameservers to query.
 
@@ -694,18 +681,14 @@ A space-separated list of resolver options to set.  Options that
 take values are specified as C<option:value>.
 
 
-=head1 IPv6 TRANSPORT
-
-The Net::DNS::Resolver library will enable IPv6 transport if the
-L<IO::Socket::IP> library package is available.
+=head1 IPv4 TRANSPORT
 
 The C<force_v4()>, C<force_v6()>, C<prefer_v4()>, and C<prefer_v6()> methods
 with non-zero argument may be used to configure transport selection.
 
 The behaviour of the C<nameserver()> method illustrates the transport
-selection mechanism.  If, for example, IPv6 is not available or IPv4
-transport has been forced, the C<nameserver()> method will only return
-IPv4 addresses:
+selection mechanism.  If, for example, IPv4 transport has been forced,
+the C<nameserver()> method will only return IPv4 addresses:
 
     $resolver->nameservers( '192.0.2.1', '192.0.2.2', '2001:DB8::3' );
     $resolver->force_v4(1);
